#ifndef	_CONFIG_
#define	_CONFIG_

#define WRITEV
#define SVR4_UCB		/* we have some of these bits too */
#define LINUX			/* more strictly Linux */

#define BSDSIGS
#define BSDLIBC			/* We have the BSD insque/remque. */
#define SWAPLIB			/* Can swap between local and net byte order */

#define SHADOW_PW		/* use shadow password file */

#define VSPRINTF

#define GETDENTS
#define NFS			/* NFS implies GETDENTS too I think. */

#define	TCP			/* has TCP/IP (of course) */
#define	SOCKETS			/*   provided by sockets */
#define BIND

#define RFINGER "/usr/bin/finger"

#endif
