/* common.h - */

/*
 * $Header: /xtel/isode/isode/h/quipu/RCS/common.h,v 9.0 1992/06/16 12:23:11 isode Rel $
 *
 *
 * $Log: common.h,v $
 * Revision 9.0  1992/06/16  12:23:11  isode
 * Release 8.0
 *
 */

/*
 *				  NOTICE
 *
 *    Acquisition, use, and distribution of this module and related
 *    materials are subject to the restrictions of a license agreement.
 *    Consult the Preface in the User's Manual for the full terms of
 *    this agreement.
 *
 */


#ifndef COMH
#define COMH

#include <isode/quipu/attrvalue.h>
#include <isode/quipu/dsp.h>
#include <isode/quipu/commonarg.h>
#include <isode/quipu/bind.h>
#include <isode/quipu/read.h>
#include <isode/quipu/compare.h>
#include <isode/quipu/abandon.h>
#include <isode/quipu/list.h>
#include <isode/quipu/ds_search.h>
#include <isode/quipu/add.h>
#include <isode/quipu/modify.h>
#include <isode/quipu/modifyrdn.h>
#include <isode/quipu/remove.h>
#include <isode/quipu/ds_error.h>

typedef struct simple_creds
	{
	DN      usc_dn;
	int     usc_passwd_len;
	char    *usc_passwd;
	char    *usc_time1;
        char    *usc_time2;
	} USC;

#endif
