/* 
 * $Header: /xtel/isode/isode/h/quipu/RCS/syntaxes.h,v 9.0 1992/06/16 12:23:11 isode Rel $
 *
 *
 * $Log: syntaxes.h,v $
 * Revision 9.0  1992/06/16  12:23:11  isode
 * Release 8.0
 *
 */

/*
 *				  NOTICE
 *
 *    Acquisition, use, and distribution of this module and related
 *    materials are subject to the restrictions of a license agreement.
 *    Consult the Preface in the User's Manual for the full terms of
 *    this agreement.
 *
 */


#ifdef PEPYPATH

#include <isode/quipu/IF-types.h>
#include <isode/quipu/SA-types.h>
#include <isode/quipu/Thorn-types.h>
#include <isode/quipu/NRS-types.h>
#include <isode/quipu/Quipu-types.h>

#else

#include <isode/pepsy/IF-types.h>
#include <isode/pepsy/SA-types.h>
#include <isode/pepsy/Thorn-types.h>
#include <isode/pepsy/NRS-types.h>
#include <isode/pepsy/Quipu-types.h>

#endif
