/* abandon.h - */

/*
 * $Header: /xtel/isode/isode/h/quipu/RCS/abandon.h,v 9.0 1992/06/16 12:23:11 isode Rel $
 *
 *
 * $Log: abandon.h,v $
 * Revision 9.0  1992/06/16  12:23:11  isode
 * Release 8.0
 *
 */

/*
 *                                NOTICE
 *
 *    Acquisition, use, and distribution of this module and related
 *    materials are subject to the restrictions of a license agreement.
 *    Consult the Preface in the User's Manual for the full terms of
 *    this agreement.
 *
 */


#ifndef QUIPUABANDON
#define QUIPUABANDON

#include <isode/quipu/dap.h>

struct ds_abandon_arg {
    int aba_invokeid;
};

#endif
